/*
 * Program:   R  A  C
 * 
 * RAC - Rows and Columns, the text extraction tool 
 * http://aurelio.net/rac
 *
 * Copyright 2003  Aurelio Marinho Jargas
 *                 Thobias Salazar Trevisan
 *
 * Please address all bugs and comments to:
 *    verde@aurelio.net
 *    thobias@thobias.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You have received a copy of the GNU General Public License along
 *  with this program, on the COPYING file.
 *
 */

#include "rac.h"


/* function defines the LINE_START, LINE_END and STEP_LINE (maybe) 
 * address of a command
 */
void get_line_range(char *command)
{
	char *p1, *temp_line, c, *temp_p1;
	int i, j=0;

	p1 = xmalloc(strlen(command)+1, "get_line_range alloc p1.");
	temp_p1 = p1;
	p1[0] = '\0';
	/* erase between '[' and ']' 
	 * now, p1 has the addresss without [xxxxx]s */ 
	for (i=0;i<strlen(command);i++){
		if (command[i] != '[' && command[i] != ']' && j == 0)
			strncat(p1,&command[i],1);
		if (command[i] == '[') j = 1;
		if (command[i] == ']') j = 0;
	}
	/* we have some line address or just a '[xxxx]' address ?! */ 
	if (p1[0] == '\0') {
		rac.LINE_START =  1;
		rac.LINE_END   = -1;
		free(temp_p1);
		return;
	}
	/* to record the chunks (chars) of read line address */
	temp_line = xmalloc(strlen(p1)+1, "get_line_range alloc tem_line.");
	temp_line[0] = '\0';
#ifdef DEBUG2
	printf("--- only lines command = %s\n",p1);
#endif
	strncpy(&c,p1,1);
	while (c) {
		/* test what kind of address.
		 * it has line range */
		if (c == ':') {
			rac.LINE_START = atoi(temp_line) == '\0' ? 1 : atoi(temp_line);
			temp_line[0] = '\0';
			p1++;
			strncpy(temp_line,p1,strlen(p1));
			rac.LINE_END = p1[0] == '\0' ? -1 : atoi(p1);
			break;
		}
		/* line step */
		else if (c == '~') {
			rac.LINE_START = atoi(temp_line);
			if (rac.LINE_START == 0)
				error("", "line start can not be zero");

			temp_line[0] = '\0';
			p1++;
			strncpy(&c,p1,1);
			rac.STEP_LINE = !c ? 0 : atoi(p1);
			rac.LINE_END = rac.STEP_LINE < 0 ? 1 : -1;
			if (rac.STEP_LINE == 0)
				error("", "STEP_LINE can not be zero");
		}
		strncat(temp_line,p1,1);
		p1++;
		strncpy(&c,p1,1);
		/* one line only ex: 'x' */
		if (!c && rac.LINE_START == 0 && rac.LINE_END == 0)
				rac.LINE_START = rac.LINE_END = atoi(temp_line);
	}

#ifdef DEBUG2
	printf("--- line_start = %li line_end = %li STEP_LINE %li\n",rac.LINE_START,rac.LINE_END,rac.STEP_LINE);
#endif
	free(temp_line);
	free(temp_p1);
}

/* check and adjust start and end points 
 * its the same to lines and columns 
 * it returns -1 if its a invalida range 
 */
int check_range(long int *start, long int *end, long int total, int range_adjust)
{
	/* convert the negative address to positive */
	if (*start < 0) *start = total + (*start+1);
	if (*end < 0)   *end   = total + (*end+1);

	if (range_adjust == RANGE_ADJUST) {
		if (*start <= total && *start >= 1) 
			*end = *end < 1 ? 1 : (*end > total ? total : *end);
		if (*end <= total && *end >= 1) 
			*start = *start < 1 ? 1 : (*start > total ? total : *start);
	}

	return (*start > total || *start < 1 || *end > total || *end < 1 ? -1 : 1);
}

/* executes (print) commands type = TYPE_COL_RANGE
 * ex:  4[1,-1,4:9],  [4:8],  ...
 */
void exec_col_range(char *command, char *col_cmd_start, char *col_cmd_end)
{
	char *token, *temp_col, *p, *temp_line, *temp_step_col;
	long int i, j, print_newline = 0;
	long int col_start, col_end, temp_col_check;
	
	/* check and adapt the line number */
	if (check_range(&rac.LINE_START, &rac.LINE_END, rac.total_lines, RANGE_NOT_ADJUST) < 1)
		return;

#ifdef DEBUG2
	printf("--- Definitive Line Range = from %li to %li\n",rac.LINE_START,rac.LINE_END);
	printf("--- col start %s end %s\n",col_cmd_start, col_cmd_end);
#endif

	 /* if we have more than 1 line (ex: '[1,-1]), than we walk (to break the column
	  * command) 'n' (number of lines) times in the col_cmd_start memory */
	temp_col = xmalloc(strlen(col_cmd_start)+1,"exec_col_range alloc temp_col.");

	for (i=rac.LINE_START; i<=rac.LINE_END; i++) {
		/* locate and get the line */
		locate_line(i);
		temp_line = get_line(i);
		strncpy(temp_col,col_cmd_start,strlen(col_cmd_start));
		temp_col[strlen(col_cmd_start)] = '\0';
		/* break the command into separate commands */
		token = strtok(temp_col, ",");
		while (token != NULL) {
			rac.STEP_COL = 0;
			/* the column command has a specific STEP_COL */
			if ((temp_step_col=strchr(token,'~')) != NULL)
				rac.STEP_COL = atoi(++temp_step_col);
			/* it is a single char or STEP_COL range */
			if ((p = strchr(token,':')) == NULL) {
				col_start = atoi(token);
				temp_col_check = rac.STEP_COL > 0 ? -1 : 1;
				if (check_range(&col_start, &temp_col_check, rac.num_chars_line[i-1]-1, RANGE_NOT_ADJUST) > 0) {
					if (rac.STEP_COL == 0)
						printf("%c",temp_line[col_start-1]);
					else
						for (j=col_start-1; j!=(temp_col_check-1)+rac.STEP_COL; j+=rac.STEP_COL) {
							/* as for uses != condition, we have to guarantee
							 * that we do not access away from the limits */
							if (col_start-1 < temp_col_check-1 && j > temp_col_check-1) break;
							if (col_start-1 > temp_col_check-1 && j < temp_col_check-1) break;
							printf("%c",temp_line[j]);
						}
					print_newline = 1;
				}
			}
			/* it is a column range */
			else {
				col_start = atoi(token) != 0 ? atoi(token) : 1;
				p++;
				col_end = atoi(p) != 0 ? atoi(p) : -1;
				if (check_range(&col_start, &col_end, rac.num_chars_line[i-1]-1, RANGE_ADJUST) > 0) {
					if (rac.STEP_COL == 0) rac.STEP_COL = col_start <= col_end ? 1 : -1;
					for (j=col_start-1; j!=(col_end-1)+rac.STEP_COL; j+=rac.STEP_COL) {
						printf("%c",temp_line[j]);
					}
					print_newline = 1;
				}
			}
			/* get the next command */
			token = strtok(NULL, ",");
		}
		free(temp_line);
		if (print_newline == 1) {
			printf("\n");
			print_newline = 0;
		}
	}
	free(temp_col);
}

/* it executes (print) commands type = TYPE_LINE_RANGE
 * ex: 2,  1~2, 3:4,  3:4[2],  1[2]:4[-1],  1:-1,  ...  
 */
void exec_line_range(char *command, char *col_cmd_start, char *col_cmd_end)
{
	long int i, j;
	long int col_start, col_end, temp_col=1;
	char *temp_line;

	/* just one line or STEP_LINE is defined
	 * so we can not adjust the range */
	if (rac.LINE_START == rac.LINE_END || rac.STEP_LINE != 0) {
		if (check_range(&rac.LINE_START, &rac.LINE_END, rac.total_lines, RANGE_NOT_ADJUST) < 1) 
			return;
	}
	else
		if (check_range(&rac.LINE_START, &rac.LINE_END, rac.total_lines, RANGE_ADJUST) < 1) 
			return;

	/* define STEP_LINE, if it is not defined */
	if (rac.STEP_LINE == 0)
		rac.STEP_LINE = rac.LINE_START < rac.LINE_END ? 1 : -1;

#ifdef DEBUG2
	printf("--- Definitive Line Range = from %li to %li\n",rac.LINE_START,rac.LINE_END);
	printf("--- STEP_LINE = %li\n",rac.STEP_LINE);
	printf("--- col start %s end %s\n",col_cmd_start, col_cmd_end);
#endif

	for (i = rac.LINE_START; i != rac.LINE_END + rac.STEP_LINE; i += rac.STEP_LINE) {
		/* as for is i != LINE_END not < or > ... and we can have a STEP_LINE
		 * greater or less than 1. 
		 * so it could try to access a line away from the limits
		 * but we can permit this access 8=) */
		if (rac.LINE_START < rac.LINE_END && i > rac.LINE_END) 	break;
		if (rac.LINE_START > rac.LINE_END && i < rac.LINE_END) 	break;

		if (i == rac.LINE_START && i == rac.LINE_END) {
			col_start = col_cmd_start == NULL ? 1 : atoi(col_cmd_start);
			col_end = col_cmd_end == NULL ? -1 : atoi(col_cmd_end);
			if (check_range(&col_start, &col_end, rac.num_chars_line[i-1]-1, RANGE_ADJUST) < 1)
				break;
			locate_line(i);
			temp_line = get_line(i);
			for (j=col_start-1;j<col_end;j++)
				printf("%c",temp_line[j]);
			printf("\n");
		}
		else if (i == rac.LINE_START) {
			col_start = col_cmd_start == NULL ? 1 : atoi(col_cmd_start);
			/* num_chars_line[i-1] - 1
			 * less 1 because we can not count new line. */
			if (check_range(&col_start, &temp_col, rac.num_chars_line[i-1]-1, RANGE_NOT_ADJUST) < 1)
				continue;
			locate_line(i);
			temp_line = get_line(i);
			for (j=col_start-1;j<rac.num_chars_line[i-1];j++)
				printf("%c",temp_line[j]);
		}
		else if (i == rac.LINE_END) {
			col_end = col_cmd_end == NULL ? -1 : atoi(col_cmd_end);
			/* num_chars less 1, we can not count new line. */
			if (check_range(&temp_col, &col_end, rac.num_chars_line[i-1]-1, RANGE_NOT_ADJUST) < 1)
				continue;
			locate_line(i);
			temp_line = get_line(i);
			for (j=0;j<col_end;j++)
				printf("%c",temp_line[j]);
			printf("\n");
		}
		else {
			locate_line(i);
			temp_line = get_line(i);
			printf("%s",temp_line);
		}
		free(temp_line);
	}
}

/* separates the column command in start and end 
 * it do not analise the address, put in memory only the columns command
 * ex: col_cmd_start = 3:4,4,-1 col_cmd_end = 3
 */
void get_col_chunk(char *command, char **col_cmd_start, char **col_cmd_end)
{
	char c, *temp_col, *temp_command = command;
	int i=0, line_key = 0;

	temp_col = xmalloc(strlen(command)+1, "get_col_chunk alloc p1.");
	temp_col[0] = '\0';
	strncpy(&c,command,1);
	while (c) {
		if (c != '[' && i == 1) strncat(temp_col,command,1);
		else if (c == ':' && i == 0) line_key = 1; 	
		if (c == ']') {
			i = 0;
			/* erase the last char, ie, ']' */
			temp_col[strlen(temp_col)-1] = '\0';
			/* is it a start column command */
			if (line_key == 0) {
				*col_cmd_start = xmalloc(strlen(temp_col)+1,
						"get_col_chunk: alloc col_cmd_start.");
				strncpy(*col_cmd_start,temp_col,strlen(temp_col)+1);
			}
			/* it is a end column command */
			else {
				*col_cmd_end = xmalloc(strlen(temp_col)+1,
						"get_col_chunk: alloc col_cmd_end.");
				strncpy(*col_cmd_end,temp_col,strlen(temp_col)+1);
			}
			temp_col[0] = '\0';
		}
		if (c == '[') i = 1;
		command++;
		strncpy(&c,command,1);
	}
	/* do not have line range. ex: '[x,y]' address, so col_cmd_start 
	 * and col_cmd_end are equal */
	if (*col_cmd_start != NULL && line_key == 0) {
		*col_cmd_end = xmalloc(strlen(temp_col)+1,
				"get_col_chunk: alloc col_cmd_end.");
		strncpy(*col_cmd_end,*col_cmd_start,strlen(*col_cmd_start)+1);
	}
	command = temp_command;
#ifdef DEBUG2
	printf("--- col_cmd_start = %s\n",*col_cmd_start);
	printf("--- col_cmd_end   = %s\n",*col_cmd_end);
#endif
	free(temp_col);
}

/* execute the comands */
void exec_cmd(char *argv[])
{
	int i, k, type;
	char **command=NULL;
	/* it has the chunk of command that 
	 * has the columns address */
	char *col_cmd_start, *col_cmd_end;

	/* get the number of independent commands (separated by ';') */
	k = read_cmd(&command,argv);

	/* it executes independent commands */
	for (i=0;i<k;i++) {
#ifdef DEBUG2
		printf("--- Command = %s\n",command[i]);
#endif
		col_cmd_start = col_cmd_end = NULL;
		rac.LINE_START = rac.LINE_END = rac.STEP_LINE = 0;

		get_line_range(command[i]);
		get_col_chunk(command[i], &col_cmd_start, &col_cmd_end);
		type = get_command_type(command[i]);
		if (type == TYPE_LINE_RANGE)
			exec_line_range(command[i], col_cmd_start, col_cmd_end);
		else if (type == TYPE_COL_RANGE)
			exec_col_range(command[i], col_cmd_start, col_cmd_end);

		if (col_cmd_start != NULL) free(col_cmd_start);
		if (col_cmd_end   != NULL) free(col_cmd_end);
		free(command[i]);
	}
	free(command);
}


