/*
 * Program:   R  A  C
 * 
 * RAC - Rows and Columns, the text extraction tool 
 * http://aurelio.net/rac
 *
 * Copyright 2003  Aurelio Marinho Jargas
 *                 Thobias Salazar Trevisan
 *
 * Please address all bugs and comments to:
 *    verde@aurelio.net
 *    thobias@thobias.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You have received a copy of the GNU General Public License along
 *  with this program, on the COPYING file.
 *
 */

#include "rac.h"


void usage(int status)
{
	if (status !=0)
		fprintf (stderr, "Tente `%s --help' para mais informaes.\n",PROGRAM_NAME);
	else {
		fprintf(stdout,"\
Uso: %s ENDEREO [ARQUIVO]\n\
 ou: %s OPES\n\
\n\
OPES\n\
\n\
     -h, --help        mostra estas informaes e sai\n\
     -V, --version     mostra a verso do programa e sai\n\
\n\
ENDEREO\n\
\n\
     Linha   Coluna    Significado\n\
     -----------------------------------------------\n\
       N       [N]     N\n\
     N;M;I   [N,M,I]   N, M e I\n\
      N:M     [N:M]    trecho de N at M\n\
      N~M     [N~M]    a partir de N, ande de M em M\n\
\n\
Se o ARQUIVO for '-', l o texto da entrada padro (STDIN).\n",PROGRAM_NAME,PROGRAM_NAME);
fprintf(stdout,"Se o nmero do endereo for negativo, conta a partir do fim.\n\
Envie relatrios de erros para: %s\n",BUG_ADDRESS);
	}
	exit (status == 0 ? 0 : 1);
}


int main(int argc, char *argv[])
{
	if (argc < 2)
		usage(1);

	/* do not use getopt, few options only */
	if ((!strcmp(argv[1],"-h")) || (!strcmp(argv[1],"--help")))
		usage(0);
	if (!strcmp(argv[1],"-V") || !strcmp(argv[1],"--version")) {
		fprintf(stdout, "%s verso %s\n", PROGRAM_NAME,VERSION);
		exit(0);
	}

	bzero(&rac,sizeof(rac));
	
	/* test if we have to read from a file or stdin */
	if (argc == 2 || (!strcmp(argv[2],"-"))) {
#ifdef DEBUG1
		printf("--- Reading from stdin\n");
#endif
		rac.stream = stdin;
		temp_file(CREATE_TEMP_FILE);
	}
	else {
		if (!(rac.stream = fopen(argv[2], "r")))
			error("", "Could not open file %s", argv[2]);
	}

	rac.total_lines = 0;
	rac.num_chars_line = NULL;

	/* get the input informations, ie, total number of line and
	 * the number of chars per line. used to 'fseek' a specific line */
	get_input_info();
#ifdef DEBUG1
	printf("--- Total lines = %li\n",rac.total_lines);
#endif

	/* execute the commands */
	exec_cmd(argv);

	free(rac.num_chars_line);
	if (fclose(rac.stream))
		error("main", "File could not be closed");

	temp_file(REMOVE_TEMP_FILE);

	exit(0);
}


