/*
 * Program:   R  A  C
 * 
 * RAC - Rows and Columns, the text extraction tool 
 * http://aurelio.net/rac
 *
 * Copyright 2003  Aurelio Marinho Jargas
 *                 Thobias Salazar Trevisan
 *
 * Please address all bugs and comments to:
 *    verde@aurelio.net
 *    thobias@thobias.org
 *
 *  This program is free software; you can redistribute it and/or modify
 *  it under the terms of the GNU General Public License as published by
 *  the Free Software Foundation, version 2.
 *
 *  This program is distributed in the hope that it will be useful,
 *  but WITHOUT ANY WARRANTY; without even the implied warranty of
 *  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *  GNU General Public License for more details.
 *
 *  You have received a copy of the GNU General Public License along
 *  with this program, on the COPYING file.
 *
 */

#include <stdio.h>
#include <stdlib.h>
#include <unistd.h>
#include <limits.h>
#include <string.h>
#include <time.h>
#include <stdarg.h>
#include <regex.h>

#define PROGRAM_NAME "rac"

#define VERSION "1.1"

#define AUTHORS "Aurelio Marinho Jargas, Thobias Salazar Trevisan"

#define BUG_ADDRESS "verde@aurelio.net, thobias@thobias.org"

#ifndef LONG_MAX
#define LONG_MAX	2147483647L
#endif

#ifndef INT_MAX
#define INT_MAX		2147483647
#endif

/* regular expression to identify the command type */
#define ER_TYPE_RANGE_COL    "^-?[0-9]*\\[([0-9:,~-]+)+\\]$"
#define ER_TYPE_RANGE_LINE   "^((-?[0-9]*(\\[[0-9-]*\\])?:-?[0-9]*(\\[[0-9-]*\\])?)|(-?[0-9]+~?-?[0-9]*))$"

/* type of command */
#define TYPE_LINE_RANGE 0
#define TYPE_COL_RANGE  1

/* used to call the temp_file function */
#define CREATE_TEMP_FILE 0
#define REMOVE_TEMP_FILE 1

#ifndef DEFAULT_TMPDIR
# define DEFAULT_TMPDIR "/tmp"
#endif

/* sufix to create the temp file (if necessary) */
#define TMPFILE_SUFIX "/rac_temp_"

/* what kind of adjustment will be done at check_range function */
#define RANGE_ADJUST      0
#define RANGE_NOT_ADJUST  1

/* command allocate memory (step) */
#define CMD_SIZE	32

/* the number of bytes per atomic read */
/* BUFFER_SIZE can not be 1. fgets reads size - 1*/
#define BUFFER_SIZE 8192

/* maximum size of error message */
#define MAX_ERROR_SIZE  256

struct struct_rac {
	/* vector used to seek lines in the input and
	 * to know how many chars each line has */
	long int *num_chars_line; 

	/* the input's total lines number*/
	long int total_lines;

	/* range line start and end */
	long int LINE_START, LINE_END;

	/* range column start and end */
	long int COL_START, COL_END;

	/* step line and step col */
	long int STEP_LINE, STEP_COL;

	FILE *stream;
	FILE *tmp;
};

struct struct_rac rac;

/* Function Prototyping */
void *xmalloc(size_t n, char string[]);		 /* lib_rac.c */
void error(char *func, char *fmt, ...);			 /* lib_rac.c */
void locate_line(long int num_line);         /* lib_rac.c */
char *get_line(long int line);				 /* lib_rac.c */
int get_command_type(char *command);		 /* lib_rac.c */
int read_cmd(char ***command, char *argv[]); /* lib_rac.c */
void get_input_info(void);    /* input.c */
void temp_file(int opt);      /* input.c */
void get_line_range(char *command);  /* exec.c */
void exec_line_range(char *command, char *col_cmd_start, char *col_cmd_end); /* exec.c */
int get_command_type(char *command); /* exec.c */
int check_range(long int *start, long int *end, long int total, int range_adjust); /* exec.c */
void get_col_chunk(char *command, char **col_cmd_start, char **col_cmd_end); /* exec.c */
void exec_cmd(char *argv[]);         /* exec.c */
void usage(int status);            /* rac.c */
int main(int argc, char *argv[]);  /* rac.c */


